#!/bin/sh
#
# Copyright (c) 2019 Vivotek Inc. All rights reserved.
#
# +-----------------------------------------------------------------+
# | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
# | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
# | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
# | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
# | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
# | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
# |                                                                 |
# | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
# | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
# | VIVOTEK INC.                                                    |
# +-----------------------------------------------------------------+
#
# Date: 2019-12-23 08:16:07
# Author: Jacko Chang (jacko.chang@vivotek.com)
#

if [ -z "${PKGROOT}" ]; then
	PKGROOT=$(realpath ./)
fi

if [ "$(which jq; echo $?)" != "0" ];then
	export PATH=${PKGROOT}/bin:${PATH}
	export LD_LIBRARY_PATH=${PKGROOT}/lib
fi

if [ -z "${SSLROOT}" ]; then
	export SSLROOT=${PKGROOT}/ssl
fi

CONFIG=${PKGROOT}/config.json

if [ ! -f ${CONFIG} ]; then
	${PKGROOT}/package.sh default 1>&2
fi

cd /

check_and_condition()
{
	result=true
	delimiter=','
	s=$@$delimiter
	while [[ "${s}" ]]; do
		key=$(echo ${s%%$delimiter*} | cut -d'=' -f1)
		expected=$(echo ${s%%$delimiter*} | cut -d'=' -f2)
		current=$(confclient -p 99 -t Value -g ${key})
		echo current ${key}=${current}, expected ${expected} >&2
		if [ ${current} != ${expected} ]; then
			result=false
			break
		fi
		s=${s#*"$delimiter"};
	done
	echo $result

}

check_conditions()
{
	LINE=${@#(COND)}
	result=false
	delimiter=';'
	s=$LINE$delimiter
	while [[ "${s}" ]]; do
		cond_result=$(check_and_condition ${s%%$delimiter*})
		if [ ${cond_result} = true ]; then
			echo ${s%%$delimiter*} - OK >&2
			result=true
			break
		fi
		s=${s#*"$delimiter"};
	done
	echo $result
}

cleanup()
{
	[ ! -z ${CONF} ] && sleep 1 && rm -f ${CONF}
}
trap "exit 1" INT TERM
trap cleanup EXIT

CONF=$(mktemp -p /mnt/ramdisk initconf-XXXXXX)

ENDPOINT=$(jq -r '.vivocloud.config' ${CONFIG})
if [ "${ENDPOINT}" = "null" ]; then
	ENDPOINT=https://support.vivocloud.com/conf
fi

AUDIO_ENABLE=$(confclient -t Value -g capability_naudioin)
while read FILE; do
	echo Download file: "${ENDPOINT}/conf/${FILE}"
	curl -sf --capath ${SSLROOT}/certs -o ${CONF} -L "${ENDPOINT}/conf/${FILE}" && break
done << EOF
$(confclient -t Value -g system_info_modelname).conf
EOF

if [ ! -f ${CONF} ]; then
	exit 0
fi
# add newline to file end to prevent config parse error
echo >> ${CONF}

# Apply initial default setting after changing mode and sleep 30 sec.
confclient -p 9 -s videoin_c0_mode=0 -c
sleep 30

( confclient -b -t ShellScript -g capability; cat ${CONF} | while read LINE; do
	case ${STATE:-NONE}${LINE} in
	NONE|NONE\;|NONE---)
		echo SKIP \"$LINE\" >&2
	;;
	NONE\;*)
		case ${LINE} in
			";(COND)"*)
				if [ $(check_conditions ${LINE#;(COND)}) = true ]; then
					echo ${LINE} - CHECK OK >&2
					STATE=NONE
				else
					echo ${LINE} - SKIP >&2
					STATE=SKIP
				fi
			;;
			"; capability_naudioin"*)
				if [ "${AUDIO_ENABLE}" == "0" ]; then
					echo ${LINE} - SKIP >&2
					STATE=SKIP
				fi
			;;
		esac
	;;
	SKIP---)
		echo SKIP end >&2
		STATE=NONE
	;;
	SKIP*)
		echo SKIP \"$LINE\" >&2
		STATE=SKIP
	;;
	APPEND---)
		echo SET $PARAM >&2
		printf 'confclient -p 9 -s "%s" -c \n' "${PARAM}"
		STATE=NONE
	;;
	APPEND*)
		echo AND \"$LINE\" >&2
		PARAM=${PARAM}\&${LINE}
	;;
	NONE*)
		echo GET \"$LINE\" >&2
		PARAM=${LINE}
		STATE=APPEND
	;;
	esac
done ) | sh

echo Done.
