#!/bin/sh
#
# Copyright (c) 2019 Vivotek Inc. All rights reserved.
#
# +-----------------------------------------------------------------+
# | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
# | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
# | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
# | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
# | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
# | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
# |                                                                 |
# | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
# | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
# | VIVOTEK INC.                                                    |
# +-----------------------------------------------------------------+
#
# Date: 2019-07-19 08:59:46
# Author: Jacko Chang (jacko.chang@vivotek.com)
#

if [ -z "${PKGROOT}" ]; then
	export PKGROOT=$(realpath ../)
	export SSLROOT=${PKGROOT}/ssl
fi

export PATH=${PKGROOT}/bin:${PATH}
export LD_LIBRARY_PATH=${PKGROOT}/lib

CONFIG=${PKGROOT}/config.json
SERVICE=${PKGROOT}/bin/service.sh

if [ ! -f ${CONFIG} ]; then
	${PKGROOT}/package.sh default 1>&2
fi

LOGPATH=/mnt/ramdisk/enable.log
PCODE_LCK=/mnt/ramdisk/pcode.lck

TEMP_CONFIG_VAL=$(mktemp -p ${PKGROOT} param-XXXXXX)
jq -r '[.vivocloud.enable, .vivocloud.apiserver, .iotbus.endpoint, .iotbus.clientid, .webrtc.deviceID] | map(tostring) | join("|")' ${CONFIG} > ${TEMP_CONFIG_VAL}
IFS="|" read SERVICE_ENABLE API_SERVER IOT_ENDPOINT CLIENT_ID DEVICE_ID < ${TEMP_CONFIG_VAL}
rm -f ${TEMP_CONFIG_VAL}

URL_TIMESTAMP=${API_SERVER}"/device/pair/v1/timestamp"
URL_PAIRPULL=${API_SERVER}"/device/pair/v1/pcodes"
URL_PAIRDEVICE=${API_SERVER}"/login.html#/login"

get_status()
{
	if [ ! -f ${PCODE_LCK} ]; then
		jq -f /dev/stdin ${CONFIG} <<- EOF
		{
			"status": "none",
			"IoTEndpoint": .iotbus.endpoint,
			"ClientID": .iotbus.clientid,
			"DeviceID": .webrtc.deviceID,
			"WebRTCEndpoint": .webrtc.endpoint
		}
		EOF
		return
	fi

	cat ${PCODE_LCK}
	if [ "$(jq -r '.status' ${PCODE_LCK})" == 'done' ]; then
		rm -f ${PCODE_LCK}
	fi
}

build_pair()
{
	TEMPPAIR=$(mktemp -p ${PKGROOT} pairgen-XXXXXX)
	pairgen -B > ${TEMPPAIR}
	echo "$(date) build pair" >> ${LOGPATH}
	cat ${TEMPPAIR} >> ${LOGPATH}
	#suspend service during pairing
	${SERVICE} suspend_for_pair
}

concat_pair()
{
	PCODE=$(jq -r '.pcode' ${TEMPPAIR})
	PCODE_ENDPOINT="${URL_PAIRDEVICE}?pcode=${PCODE}"
	if [ -z ${PCODE} ]; then
		PCODE_STATUS="fail"
		PCODE_ENDPOINT=""
	else
		PCODE_STATUS="pulling"
	fi

	cat <<- EOF | jq -s '.[0] * .[1]' - ${TEMPPAIR} > ${PCODE_LCK}
	{
		"status":"${PCODE_STATUS}",
		"IoTEndpoint": "${IOT_ENDPOINT}",
		"ClientID": "${CLIENT_ID}",
		"DeviceID": "${DEVICE_ID}",
		"endpoint":"${PCODE_ENDPOINT}",
		"expiration": $(($(date +%s)+600))
	}
	EOF
	rm -f ${TEMPPAIR}
}

pull_info()
{
	TEMP_PCODE_VAL=$(mktemp -p ${PKGROOT} param-XXXXXX)
	jq -r '[.pcode, .expiration] | map(tostring) | join("|")' ${PCODE_LCK}   > ${TEMP_PCODE_VAL}
	IFS="|" read PCODE EXPIRATION < ${TEMP_PCODE_VAL}
	rm -f ${TEMP_PCODE_VAL}

	while [ -f ${PCODE_LCK} ]; do
		CUR_PCODE=$(jq -r '.pcode' ${PCODE_LCK})
		if [ -z ${CUR_PCODE} ]; then
			rm -f ${PCODE_LCK}
			exit 0
		fi

		if [ ${SERVICE_ENABLE} = "false" ]; then
			rm -f ${PCODE_LCK}
			exit 0
		fi

		if [ ${CUR_PCODE} != "${PCODE}" ]; then
			exit 0
		fi

		CUR_TIME=$(date +%s)
		if [ "${CUR_TIME}" -ge "${EXPIRATION}" ]; then
			rm -f ${PCODE_LCK}
			TIMEOUT=true
			break
		fi

		RES=$(curl -kH "Content-Type:application/json" --request GET -s "${URL_PAIRPULL}/${PCODE}")
		echo "$(date) pull info" >> ${LOGPATH}
		echo "${RES}" | jq 'del(.private, .cert)' >> ${LOGPATH}
		STATUS=$(echo "${RES}" | jq -r ".status")
		if [ "${STATUS}" == "0" ]; then
			# import cert / private key
			keyring rm ${SSLROOT}/awsiot/IoTPrivateKey.pem
			mkdir -p ${SSLROOT}/awsiot
			echo "${RES}" | jq -r '.cert' > ${SSLROOT}/awsiot/IoTCertificate.crt
			echo "${RES}" | jq -r '.private' > ${SSLROOT}/awsiot/IoTPrivateKey.pem
			keyring im ${SSLROOT}/awsiot/IoTPrivateKey.pem

			# update config.json
			TEMPCONF=$(mktemp -p ${PKGROOT} config-XXXXXX)
			echo "${RES}" | jq '{iotbus: {clientid: .deviceID, endpoint: .ioTEndpoint}, webrtc: {deviceID: .deviceID, endpoint: .signalServer}, cloudrecd: {cameraID: .deviceID, identityID: .userKey, ioTCredentialEndpoint: .ioTCredentialEndpoint}}' > ${TEMPCONF}
			jq -s '.[0] * .[1]' ${CONFIG} ${TEMPCONF} > ${CONFIG}~
			rm -f ${TEMPCONF}
			mv ${CONFIG}~ ${CONFIG}

			# pair done change status
			jq '.status=$status' --arg status "done"  ${PCODE_LCK} > ${PCODE_LCK}~
			mv ${PCODE_LCK}~ ${PCODE_LCK}
			${PKGROOT}/bin/initconf.sh &> /dev/null &
			${PKGROOT}/package.sh start &> /dev/null &
			break
		fi
		sleep 5
	done
	[ "${TIMEOUT}" == true ] && ${SERVICE} resume_for_pair
}

send_json()
{
	printf "Content-type: application/json; charset=utf-8\r\n"
	printf "\r\n"
	jq -c '.' ${1}
	printf "\r\n"
}

send_msg()
{
	printf "Content-type: text/html\r\n"
	printf "\r\n"
	echo ${*}
	printf "\r\n"
}

case ${REQUEST_METHOD} in
	GET)
		get_status | send_json -
		;;
	POST)
		build_pair
		concat_pair
		get_status | send_json -
		#close stdout/stderr
		exec 1>&-
		exec 2>&-
		# open stdout/stderr
		exec 1>&0
		exec 2>&0
		pull_info
		;;
	PUT)
		;;
	*)
		send_msg "ERROR: Unknown METHOD." ${REQUEST_METHOD}
		exit 1
		;;
esac

exit 0
